@ECHO OFF
ECHO SQL Server Database Maintainance
ECHO Written by: Jason Faulkner
ECHO SysadminGeek.com
ECHO.
ECHO.

SETLOCAL EnableExtensions EnableDelayedExpansion

IF {%1}=={} GOTO Help
IF {%1}=={?} GOTO Help
IF {%1}=={-?] GOTO Help
IF {%1}=={/?} GOTO Help

SET Database=%~1
SET Server=%ComputerName%
SET User=n/a
SET Pass=n/a

:ProcessParams
IF {%2}=={} GOTO End

REM Once an action param is encountered, exit the config
IF /I {%2}=={/COMPACT} GOTO DoMaint
IF /I {%2}=={/REINDEX} GOTO DoMaint
IF /I {%2}=={/VERIFY} GOTO DoMaint

SET ParamValue=%~2

REM Check for server specification
SET CheckParam=%~2
SET CheckParam=%CheckParam:~0,3%
IF /I {%CheckParam%}=={/S:} SET Server=%ParamValue:~3,99%

REM Check for user name specification
SET CheckParam=%~2
SET CheckParam=%CheckParam:~0,3%
IF /I {%CheckParam%}=={/U:} SET User=%ParamValue:~3,99%

REM Check for password specification
SET CheckParam=%~2
SET CheckParam=%CheckParam:~0,3%
IF /I {%CheckParam%}=={/P:} SET Pass=%ParamValue:~3,99%

SHIFT /2
GOTO ProcessParams

:DoMaint
REM Set credentials
SET Login=-E
IF NOT {%User%}=={n/a} SET Login=-U %User% -P %Pass%
SET SQLConnection=-S "%Server%" %Login%

ECHO Server:   %Server%
ECHO Database: %Database%
ECHO.
ECHO [%DATE% @ %TIME%]
ECHO.
:ActionParams
IF {%2}=={} GOTO End

IF /I {%2}=={/COMPACT} (
	ECHO Compacting database...
	SQLCMD %SQLConnection% -Q "DBCC ShrinkDatabase ('%Database%',10)"
)

IF /I {%2}=={/REINDEX} (
	SET ReindexScript="%TEMP%\Reindex_%RANDOM%.sql"
	ECHO --Reindex DB Script > !ReindexScript!
	ECHO DECLARE @Table varchar(255^);DECLARE TableList CURSOR FOR SELECT [name] FROM sys.Tables ORDER BY [name] ASC; >> !ReindexScript!
	ECHO OPEN TableList;FETCH NEXT FROM TableList INTO @Table; >> !ReindexScript!
	ECHO WHILE @@FETCH_STATUS=0 BEGIN; >> !ReindexScript!
	ECHO EXEC('ALTER INDEX ALL ON [' + @Table + '] REBUILD WITH (FILLFACTOR = 100^)'^);FETCH NEXT FROM TableList INTO @Table;END; >> !ReindexScript!
	ECHO CLOSE TableList;DEALLOCATE TableList; >> !ReindexScript!

	ECHO Reindexing database...
	SQLCMD %SQLConnection% -d "%Database%" -i !ReindexScript!
	
	IF EXIST !ReindexScript! DEL /F /Q !ReindexScript!
)

IF /I {%2}=={/VERIFY} (
	ECHO Checking database integrity...
	ECHO No messages after this line is good.
	SQLCMD %SQLConnection% -Q "DBCC CheckDB ('%Database%') WITH No_InfoMsgs"
)

ECHO Done.
ECHO --------------------
ECHO.
ECHO [%DATE% @ %TIME%]
ECHO.

SHIFT /2
GOTO ActionParams

:Help
ECHO Performs routine maintenance tasks on a SQL Server database.
ECHO.
ECHO Usage:
ECHO  %~n0 Database [Config Options] [Actions]
ECHO.
ECHO Config Options - Specify 0 or more in any order.
ECHO  /S:Server   : SQL Server to connect to.
ECHO                Default Value = Default SQL instance on the local machine.
ECHO  /U:UserName : SQL Account to connect to the SQL Server with.
ECHO                Must be used in conjuction with /P:
ECHO                If no UserName is provided then a trusted connection is used.
ECHO  /P:Password : Password of the respective SQL Account.
ECHO.
ECHO Actions - Specify 1 or more in the order you want them processed.
ECHO  /COMPACT : Compact the database to free disk space.
ECHO  /REINDEX : Rebuild the internal SQL indexes which may increase performance.
ECHO  /VERIFY  : Perform internal error checking and report any errors.

:End
ENDLOCAL